# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-05-09 16:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserMeta',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.SlugField(help_text=b'The ID name of this bit of data. Should be only lowercase numbers and letters, dashes or underscores. No spaces.')),
                ('value', models.TextField(help_text=b'The value, data or code for this item')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='meta', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'user metadata',
            },
        ),
        migrations.AlterUniqueTogether(
            name='usermeta',
            unique_together=set([('user', 'key')]),
        ),
    ]



from django.test import TestCase
from django.db import IntegrityError

from .models import User

import pprint
pp = pprint.PrettyPrinter(indent=4)


class TestUserMeta(TestCase):
    def test_create_meta(self):
        user = User.objects.create(username="bobby", email="bobby@dot.com")

        user.meta.create(key="favorite_animal", value="penguin")
        user.meta.create(key="some_important_id", value=123456789)

        #pp.pprint(user.__dict__)

        #pp.pprint(user.meta.values())

        self.assertEqual(
            unicode(user.meta.get(key="favorite_animal")),
            "penguin")

        self.assertEqual(
            unicode(user.meta.get(key="some_important_id")),
            "123456789")

        try:
            user.meta.create(key="favorite_animal", value="penguin")
        except IntegrityError:
            pass

    def test_meta_shortcuts(self):
        user = User.objects.create(username="bobby", email="bobby@dot.com")

        user.meta['favorite_animal'] = "penguin"
        user.meta['some_important_id'] = 123456789

        self.assertEqual(
            user.meta['favorite_animal'],
            "penguin")

        self.assertEqual(
            user.meta['some_important_id'],
            "123456789")

        self.assertIn('favorite_animal', user.meta)

        del user.meta['favorite_animal']

        self.assertNotIn('favorite_animal', user.meta)

        self.assertEqual(
            user.meta['favorite_animal'],
            None)



from django.contrib.auth.models import User
from django.contrib.auth.admin import UserAdmin as BuiltinUserAdmin
from django.contrib import admin

from .models import UserMeta

admin.site.unregister(User)


class UserMetaInline(admin.TabularInline):
    model = UserMeta
    extra = 0


class UserAdmin(BuiltinUserAdmin):
    inlines = [
        UserMetaInline,
    ]

admin.site.register(User, UserAdmin)



from django.db import models
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist


class MetaManager(models.Manager):
    """
    A related model manager for meta models. Allows you to treat
    the `user.meta` attribute like a dictionary.
    """
    use_for_related_fields = True

    def __getitem__(self, key):
        try:
            item = self.get(key=key)
            return item.value
        except ObjectDoesNotExist:
            return None

    def __setitem__(self, key, value):
        item, created = self.get_or_create(
            key=key, defaults={'value': value})
        if not created and item.value != value:
            item.value = value
            item.save()

    def __delitem__(self, key):
        try:
            item = self.get(key=key)
            item.delete()
        except ObjectDoesNotExist:
            pass

    def __contains__(self, item):
        try:
            item = self.get(key=item)
            return True
        except ObjectDoesNotExist:
            return False


class ModelMeta(models.Model):
    """
    Abstract model for storing key-value metadata. A model would extend
    this and define a foreign key to another model. Metadata are totally
    arbitrary bits of text that describe the model in some way.
    """
    key = models.SlugField(
        help_text="The ID name of this bit of data. Should be only "
        "lowercase numbers and letters, dashes or underscores. No spaces.")
    value = models.TextField(
        help_text="The value, data or code for this item")

    objects = MetaManager()

    def __unicode__(self):
        return unicode(self.value)

    def __str__(self):
        return str(self.value)

    class Meta:
        abstract = True


class UserMeta(ModelMeta):
    """
    Adds a meta data attribute to the builtin user object
    """
    user = models.ForeignKey(User, related_name='meta')

    class Meta:
        unique_together = ('user', 'key')
        verbose_name_plural = "user metadata"
        app_label = 'metamodel'



from setuptools import setup, find_packages

setup(
    name="django-metamodel",
    packages=find_packages(),
    version="1.1.0",
    author="Ryan Mark",
    author_email="ryan@mrk.cc",
    description="Add arbitrary data to your django users. This app adds `user.meta` which acts like a dictionary and stores whatever the hell you want. But you could add `.meta` to any of your models.",
    url="http://github.com/ryanmark/django-metamodel/",
    license="MIT",
    install_requires=["django>=1.4", ],
)
